function [cx,cy,ra,rb,ang]=cmodel1(sys,Bs,pos,par)
%CMODEL Camera model that generates centres, orientations, semi-axes of 
% the projection of the control circles stored in Bs, with instrinsic, 
% extrinsic camera parameters par and pos. 
% derived from 
%   Version 3.0  10-17-00
%   Janne Heikkila, University of Oulu, Finland
%
%Usage:
%   [cx,cy,a,b,ang] = cmodel(sys,Bs,pos,par)
%
%where       
%   sys  = system configuration information (see configc.m) 
%   Bs   = 3-D configuration of the control points (position, orientation, radius)
%   pos  = camera position and orientation
%   par  = camera intrinsic parameters:
%          par(1) = scale factor ~1
%          par(2) = effective focal length
%          par(3:4) = principal point
%          par(5:6) = radial distortion coefficients
%          par(7:8) = tangential distortion coefficients
%   cx,cy   = centres of the ellipses projected from control circles
%   a,b,    = semi- major and minor axes
%   ang     = angles of the ellipses to horizontal axis



NDX=sys(1); NDY=sys(2); Sx=sys(3); Sy=sys(4);

if length(pos)~=6
  error('Position vector should contain [x y z w p r].');
end

if length(par)~=8
  error('Parameter vector should be 1 x 8 matrix.');
end

if size(Bs,1)~=16
  error('Control point matrix should have 16 rows');
end

n=size(Bs,2);
Asp=par(1); Foc=par(2);
Cpx=par(3); Cpy=par(4);
Rad1=par(5); Rad2=par(6);
Tan1=par(7); Tan2=par(8);

M=eye(4);
wa=pos(4)*pi/180;
pa=pos(5)*pi/180;
ra=pos(6)*pi/180;
cw=cos(wa); sw=sin(wa);
cp=cos(pa); sp=sin(pa);
cr=cos(ra); sr=sin(ra);

M(1,:)=[cr*cp -sr*cw+cr*sp*sw sr*sw+cr*sp*cw pos(1)];
M(2,:)=[sr*cp cr*cw+sr*sp*sw -cr*sw+sr*sp*cw pos(2)];
M(3,:)=[-sp cp*sw cp*cw pos(3)];

P=[Foc 0 0 0;0 Foc 0 0;0 0 1 0];

A=P*M;
B1=Bs(1:4,:);
B2=Bs(5:8,:);
B3=Bs(9:12,:);
B4=Bs(13:16,:);

H=B1*A(3,1)+B2*A(3,2)+B3*A(3,3)+B4*A(3,4);
q1=A(1,:)*H;
q2=A(2,:)*H;
q3=A(3,:)*H;
x=(q1./q3)';
y=(q2./q3)';

% get a,b, ang
for i= 1:size(Bs,2)
    % get a,b, ang for each circle seperately
    G = reshape(Bs(:,i),4,4)';
    S = inv(A*G*A'); % S is the quadric matrix in image plane
%     [V,D] = eig(S);
%     [ds,idx] = sort(diag(D),'ascend');
%     a(i) = sqrt(-1/ds(1));
%     b(i) = sqrt(-1/ds(2));
%     t1 = V(idx)*[1 0 1]';
%     ang(i) = atan2(t1(2)/t1(3),t1(1)/t1(3)); 

% below is according Mathworld definition
    a = S(1,1);
    b = S(1,2);
    c = S(2,2);
    d = S(1,3);
    f = S(2,3);
    g = S(3,3);
    
    ra(i) = sqrt(2*(a*f^2+c*d^2+g*b^2-2*b*d*f-a*c*g)/...
        (b^2-a*c)/(sqrt((a-c)^2+4*b^2)-a-c));
    rb(i) = sqrt(2*(a*f^2+c*d^2+g*b^2-2*b*d*f-a*c*g)/...
        (b^2-a*c)/(-sqrt((a-c)^2+4*b^2)-a-c));
    if b == 0
        if a<c
            ang(i) = 0; 
        else
            ang(i) = pi/2;
        end
    else
        if a<c
            ang(i) = atan(2*b/(a-c))/2; 
        else
            ang(i) = atan(2*b/(a-c))/2+pi/2;
            %ang(i) = pi/2;
        end
    end

end


% consider nonlinear distortion now
r2=x.*x+y.*y;
delta=3*Rad1*r2+5*Rad2*r2.*r2;

xn=x.*(1+delta)+6*Tan1*x.*y-Tan2*(r2-6*x.*x); 
yn=y.*(1+delta)-Tan1*(r2-6*y.*y)+6*Tan2*x.*y; 
Q=4*Rad1*r2+6*Rad2*r2.*r2+8*Tan1*y+8*Tan2*x+1;
xn=xn./Q; yn=yn./Q;
  
cx=NDX*Asp*xn/Sx+Cpx;
cy=NDY*yn/Sy+Cpy;

ra = ra*NDX*Asp/Sx;
rb = rb*NDY/Sy;

ra = ra(:);rb = rb(:);ang = ang(:);
